// Copyright (c) 2023 Dominic Masters
// 
// This software is released under the MIT License.
// https://opensource.org/licenses/MIT

#include "game/Game.hpp"
#include "scene/Scene.hpp"

using namespace Dawn;

IGame::IGame() {

}

void IGame::init() {
  auto selfAsGame = this->getSelfAsGame();

  renderHost = std::make_shared<RenderHost>();
  renderHost->init(selfAsGame);
  
  assetManager = std::make_shared<AssetManager>();
  assetManager->init();
  
  localeManager = std::make_shared<LocaleManager>();
  localeManager->init(selfAsGame);

  inputManager.init(selfAsGame);
  saveManager.init(selfAsGame);

  this->initManagers();

  auto initialScene = this->getInitialScene();
  nextFrameScene = std::make_shared<Scene>(selfAsGame, initialScene);
}

void IGame::update() {
  this->assetManager->update();
  this->inputManager.update();

  if(nextFrameScene) {
    nextFrameScene->stage();
    currentScene = nextFrameScene;
    nextFrameScene = nullptr;
  }

  timeManager.update();
  if(currentScene) currentScene->update();
  renderHost->update(this->getSelfAsGame());
}

bool_t IGame::isCloseRequested() {
  return (
    renderHost->isCloseRequested()
  );
}

std::shared_ptr<Scene> IGame::getCurrentScene() {
  return currentScene;
}

std::shared_ptr<Game> IGame::getSelfAsGame() {
  return std::static_pointer_cast<Game>(shared_from_this());
}

IGame::~IGame() {
  currentScene = nullptr;
  nextFrameScene = nullptr;
  assetManager = nullptr;
  renderHost = nullptr;
}